
//---------------------------------------------------------
/**    @file	BlobbySurface.h
*     @brief	BlobbySurface.h is the header for CLASS
                BlobbySurface.cpp								*/
//---------------------------------------------------------

#ifndef BlobbySurface_h
#define BlobbySurface_h

#include "Surface.h"
#include "MeshSurface.h"

#define DEFAULT_BLOBBYNESS -2.5
#define DEFAULT_CUTOFF 6
/** @brief This class builds the blobby surface, triangulate it and the use mesh surface routines
for the rest of the work.

@author Sergio Decherchi 
@date 10/01/2012
*/
class BlobbySurface: public MeshSurface
{
private:
	/** blobbyness*/
	double B;
	/** cut-off distance in Angstrom to speed-up computations. It is computed based on B value.
	The bigger the gaussian the less the cut-off*/
	double cutoff;
	
public:
	/** Default constructor*/
	BlobbySurface();
	/** set DelPhi environment*/
	BlobbySurface(DelPhiShared* ds);			

	//////////////////////// INTERFACE MANDATORY METHODS /////////////////////////////////
	/** Compute blobby mesh and run MeshSurface::build()*/
	virtual bool build();
	/** Print number a summary of the blobby*/
	virtual void printSummary();		
	/////////////////////////////////////////////////////////////

	//////////////////////////// BLOBBY SURFACE METHODS ///////////////////////////////
	void setBlobbyness(double b);
	double getBlobbyness();

	virtual ~BlobbySurface();
};

#endif