
#ifndef  DelphiShared_h
#define  DelphiShared_h

#include "globals.h"
#include "tools.h"

#define DEFAULT_PERFIL 80
#define DEFAULT_SCALE 2.0

// Status map codes
#define STATUS_CAVITY_POINT_SHAPE_UNDER_CHECK -10
#define STATUS_CAVITY_POINT_SHAPE_OK -5
#define STAUTS_POINT_INSIDE -1
#define STATUS_POINT_TEMPORARY_OUT 2
#define STATUS_POINT_OUT 3

/** @brief This class is the DelPhi Shared variables environment which emulates the DelPhi environment.
*/
class DelPhiShared {   
public:
	DelPhiShared();
	DelPhiShared(double scale,double perfill, char* fn,bool map,bool status);
	
	/** Fortran DelPhi binding function. Allows to link DelPhi old code to
	    the new surface framework */
	void DelPhiBinding(	double xmin,double ymin,double zmin,
						double xmax,double ymax,double zmax,
						double c1,double c2,double c3,double rmax,double perf,
						int* local_i_epsmap,int igrid,double scale,
						double* local_i_scspos,double* local_i_scsnor,
						bool* local_i_idebmap,int* local_i_ibgp,int maxbgp);

	/**If in DelPhi binding mode, call it before denstructor. It will be change the
	variables in a Fortran/DelPhi compatible mode and will avoid the denstruction of the variables
	passed to DelPhi. Always call this function ONLY just before the denstructor of the DelPhi shared
	environment*/
	void finalizeBinding();
	/** Save the epsmaps in a format such that DelPhi can read them*/
	void saveEpsMaps(char* fname);
	/** Save the boundary grid points in a format such that DelPhi can read them*/
	void saveBGP(char* fname);
	void saveStatus(char* fname);
	bool clearAndAllocEpsMaps();
	void clearEpsMaps();
	bool loadAtoms(char fn[BUFLEN]);
	bool loadAtoms(int na,double* pos,double* r,double* q,int* d);
	/** Emulates DelPhi grid construction*/
	bool buildGrid(double scale,double perfill);
	bool getDelphiBinding()
	{
		return delphiBinding;
	}

	void buildEpsmap(bool a)
	{
		buildEpsMap = a;
	}

	void buildStatusmap(bool a)
	{
		buildStatus = a;
	}

	double scale;
	double perfill;
	bool delphiBinding;
	bool buildEpsMap;
	bool buildStatus;
	// epsmap is a four dimensional matrix
	int *epsmap,*ibgp;
	bool *idebmap;
	int maxbgp;
	double *scsnor,*scsarea,*scspos;
	// face area = side*side;
	double A;	
	double* x;
	double* y;
	double* z;
	int nbgp;
	double side;
	int nx,ny,nz;
	// the status matrix is the status for each grid point:
	// -5 -> cavity point is inside a well shaped cavity (temporary)
	// -10 -> cavity point under shape check (temporary)
	// -1 -> point is inside
	// 2 -> point is temporary outside (may be really out or in cavity)
	// 3 -> point is really out
	// 4 -> point is in cavity number 1
	// 5 -> point is in cavity number 2
	// 6 -> point is in cavity number 3
	// and so on....
	// Thus 65536-4 cavities can be detected at best; it should be enough...
	short*** status;
	int numCavities;
	double rmaxdim;
	double xmin,xmax,ymin,ymax,zmin,zmax;
	double baricenter[3];
	// atoms vector. Pointer to pointers to atoms
	Atom** atoms;
	// num atoms
	int numAtoms;
	char file[BUFLEN];
	double hside;
	/** dynamical vector of cavities*/
	vector < vector <int* >* >* cavitiesVec;
	/** dynamical vector of cavities size*/
	vector < double> cavitiesSize;
	/** dynamical vector of cavities filling*/
	vector <bool> cavitiesFlag;

	virtual ~DelPhiShared();
}; 

#endif